<?php

namespace Cacbot;

class Comment{


    /**
     * Filters a comment’s data before it is inserted into the database.
     * Sets the interlocutor if it isn't set already.
     */
    public static function enable()
    {
        \add_filter('preprocess_comment', [self::class, 'handle_preprocess_comment']);
        \add_action('comment_post', [self::class, 'handle_comment_post']);
    }

    public static function handle_comment_post(int $comment_id)
    {
        global $CacbotProtocol;
        if ($CacbotProtocol !== "remote_node") {
          return;
        }

        $Comment = \get_comment($comment_id);

        // The plugin only responds to comments made on an Cacbot conversation CPT or posts with specific meta
        $post_type = \get_post_type($Comment->comment_post_ID);
        $is_anchor_post = \get_post_meta($Comment->comment_post_ID, '_cacbot_anchor_post', true) === true;

        if ($post_type !== "cacbot-conversation" && !$is_anchor_post) {
            return;
        }

        self::do_process_comment($Comment);
    }


    /**
     * Handles the preprocess_comment filter for Cacbot Conversations.
     * Sets the interlocutor if the comment is on a 'cacbot-conversation' post type
     * or if the post has "_cacbot_anchor_post" meta data set to true.
     *
     * @param array $comment_data Comment data array.
     * @return array Filtered comment data.
     */
    public static function handle_preprocess_comment($comment_data)
    {
        $post_id = $comment_data['comment_post_ID'];
        if (!Conversation::is_cacbot_conversation($post_id)) {
            return $comment_data;
        }
        $post_id = AnchorPost::filter_for_linked_post_id($post_id, $comment_data['user_id']);

        //At this point, the comment is actually on a Cacbot Conversation.
        if (metadata_exists('post', $post_id, PREFIX . "interlocutor_user_id")) {
            $Interlocutor_user_id = \get_post_meta($post_id, PREFIX . "interlocutor_user_id", true);
        }

        if (metadata_exists('post', $post_id, PREFIX . "respondent_user_id")) {
            $Respondent_user_id = \get_post_meta($post_id, PREFIX . "respondent_user_id", true);
        }

        $published_comment_count = self::get_published_comments_count($post_id);

        // if first comment
        if ($published_comment_count === 0) {
            if (isset( $Interlocutor_user_id )) {
                if ($comment_data['user_id'] != $Interlocutor_user_id) {
                    \wp_die("You are not allowed to do that. Comment.class.php(67)");
                }
            } else {
                \update_post_meta($post_id, PREFIX . "interlocutor_user_id", $comment_data['user_id']);
                if (!User::is_user_a_cacbot($comment_data['user_id'])) {
                    \update_post_meta($post_id, PREFIX . "respondent_user_id", User::get_cacbot_assistant_user_id());
                }
            }
        }
        return $comment_data;
    }

    /**
     * Get the number of published comments for a post.
     *
     * @param int $post_id The ID of the post.
     * @return int The number of published comments.
     */
    public static function get_published_comments_count($post_id)
    {
        // Make sure the post_id is an integer
        $post_id = (int)$post_id;

        // Use the WordPress get_comments() function to retrieve comments
        $comments_count = \get_comments(array(
            'post_id' => $post_id,
            'status' => 'approve', // Only count approved (published) comments
            'count' => true       // Only return the count of comments
        ));

        return $comments_count;
    }



    public static function do_process_comment($Comment)
    {
        $comment_ID = $Comment->comment_ID;
        $Prompt = new Prompt();
        $Prompt->init_this_prompt($comment_ID, "created on remote");
        $Prompt->open_ai_api_key = ApiKey::set_open_ai_api_key_in_Prompt($comment_ID);
        \add_comment_meta($comment_ID, "Prompt_being_sent", \var_export($Prompt, true));
        $next_speaker_user_id = Interlocutor::determine_next_speaker($Prompt);
        $BandwidthManager = new BandwidthManager();
        $Prompt = $BandwidthManager->manage_pre_send_Prompt($Prompt);
        $Prompt->response = $Prompt->send_up();
        \add_comment_meta($comment_ID, "response_body", \var_export($Prompt->response['body'], true));
        if (isset($Prompt->model)) {
            if ($Prompt->model === "dall-e-3") {
                $comment_content = Dall_E_3::doHandleResponse($Prompt);
                self::put_comment_reply_on_post($next_speaker_user_id, $Prompt->post_id, $comment_content);
                return;
            }
        } else {
            $response = self::correctEncoding($Prompt->response['body']);
            $comment_id = self::put_comment_reply_on_post($next_speaker_user_id, $Prompt->post_id, $response);
            \update_comment_meta($comment_id, "incoming_response", \var_export($Prompt->response['body'], true));
        }
        $Prompt = $BandwidthManager->manage_post_send_Prompt($Prompt);
        return $response;
    }

    /**
     * Insert a comment reply on a post.
     *
     * @param int $comment_author_user_id The user ID of the comment author.
     * @param int $post_id The ID of the post.
     * @param string $comment_content The content of the comment.
     * @return int The ID of the inserted comment.
     * @throws \Exception If an error occurs while posting the comment.
     */
    public static function put_comment_reply_on_post($comment_author_user_id, $post_id, $comment_content)
    {
        // $post_author_id = \get_post_field('post_author', $post_id);
        $user = \get_userdata($comment_author_user_id);
        $nicename = $user->display_name;
        $comment_content = str_replace('```', '###TRIPLE_BACKTICK###', $comment_content);
        $comment_data = array(
            'comment_post_ID' => $post_id,
            'comment_author' => $nicename,
            'comment_author_email' => $user->user_email,
            'comment_author_url' => "https://cacbot.com",
            'comment_content' => $comment_content,
            'comment_type' => 'comment',
            'comment_parent' => 0,
            'user_id' => $comment_author_user_id,
            'comment_date' => current_time('mysql'),
            'comment_approved' => 1,
        );
        $comment_id = \wp_insert_comment($comment_data);

        if ($comment_id) {
            return $comment_id;
        } else {
            throw new \Exception("An error occurred while posting the comment.");
        }
    }

    private static function correctEncoding(string $string): string
    {
        if (strlen($string) <= 3) {
            return $string;
        }
        $string = substr($string, 1, -1);
        $string = str_replace('\n', '
', $string);
        return $string;
    }

    /**
     * Set comment meta data from HTTP parameters.
     *
     * @param int $comment_id The ID of the comment.
     * @return void
     */
    public static function xxxset_comment_meta_from_http_parameters($comment_id)
    {

        $parameters = [
            "functions",
            "account_user_id",
            "status",
            "completion_tokens",
            "max_tokens",
            "total_tokens",
            "prompt_tokens",
            "model",
            "open_ai_api_key",
            "remote_open_ai_api_key",
            "system_instructions",
            "reply_strategy"
        ];
        foreach ($parameters as $parameter) {
            if (isset($_REQUEST[$parameter])) {
                $meta_key = self::$meta_key . $parameter;
                \update_comment_meta($comment_id, $meta_key, $_REQUEST[$parameter]);
            }
        }
    }

    /**
     * Set the OpenAI API key in the comment meta if it does not already exist. When this function is run,
     * SettableHTTP_Properties::set_comment_meta_from_http_parameters should have already been run, therefore if a key
     * was posted in the HTTP request it should already be in the comment meta.
     *
     * @param int $comment_ID The ID of the comment.
     * @return string the api key
     */
    public static function set_open_ai_api_key_in_comment_meta($comment_ID)
    {
        $meta_key = "_cacbot_open_ai_api_key";
        $api_key = \get_comment_meta($comment_ID, $meta_key, true);

        if (empty($api_key)) {
            $api_key = ApiKey::get_openai_api_key();
            if ($api_key) {
                \update_comment_meta($comment_ID, $meta_key, $api_key);
            }
        }
        return $api_key;
    }

    /**
     * Function to intercept a comment before it is published.
     *
     * @param array $comment_data The comment data array.
     * @return array The modified comment data array.
     */
    public static function xxx($comment_data)
    {
        global $CacbotProtocol;

        //if ("remote_node" === $CacbotProtocal) {
        // Check if the post type is 'cacbot-conversation'
        if (\get_post_type($comment_data['comment_post_ID']) !== "cacbot-conversation") {
            return $comment_data;
        }
        Interlocutor::set_interlocutor($comment_data);
        // }
        return $comment_data;
    }

    /**
     * Retrieves the content of the latest approved comment on a given post.
     *
     * @param int $post_id The ID of the post for which to retrieve the latest comment.
     * @return string|bool The content of the latest comment if available, or false if there are no comments.
     */
    public static function get_latest_comment_content($post_id)
    {
        // Get the latest comment for the given post ID
        $comments = \get_comments([
            'post_id' => $post_id,
            'number' => 1,
            'orderby' => 'comment_date',
            'order' => 'DESC',
            'status' => 'approve' // Only fetch approved comments
        ]);

        // Check if there are any comments
        if (!empty($comments) && isset($comments[0])) {
            return $comments[0]->comment_content;
        }

        // Return false if no comments are found
        return false;
    }

    public static function change_comment_form_placeholder_text($post_id)
    {
        // Check if custom placeholder text metadata exists
        if (\metadata_exists('post', $post_id, '_cacbot_comment_form_placeholder_text')) {
            $placeholder_text = get_post_meta($post_id, '_cacbot_comment_form_placeholder_text', true);

            // Add filter to modify comment form defaults
            \add_filter('comment_form_defaults', function ($defaults) use ($placeholder_text) {
                $defaults['comment_field'] = '<p class="comment-form-comment"><label for="comment">'
                    . _x('Comment', 'noun') . '</label><textarea id="comment" name="comment" placeholder="'
                    . esc_attr($placeholder_text)
                    . '" cols="45" rows="8" aria-required="true"></textarea></p>';
                return $defaults;
            });
        }
    }

}
