<?php

namespace Cacbot;

/**
 * Class RemotePost
 *
 * Provides methods for interacting with remote WordPress posts and metadata through REST API.
 *
 * @package Cacbot
 */
class RemotePost {

    /**
     * Sends a request to a specified route on a remote WordPress site's REST API.
     *
     * @param string $remote_url The base URL of the remote WordPress site (e.g., 'https://example.com').
     * @param string $remote_user_name The username for authenticating with the remote site.
     * @param string $remote_wp_application_password The application password for the remote user.
     * @param string $remote_route The route on the remote WordPress REST API (e.g., '/wp/v2/posts').
     * @param array  $payload The data to send in the request body, as an associative array.
     * @param string $method HTTP method to use for the request (e.g., 'POST', 'DELETE').
     *
     * @return array An associative array containing:
     *               - 'success' (bool): Whether the request was successful.
     *               - 'message' (string): Error message if unsuccessful.
     *               - 'response' (array|null): The decoded JSON response if successful.
     */
    public static function send_request_to_remote_wp_api($remote_url, $remote_user_name, $remote_wp_application_password, $remote_route, $payload = [], $method = 'POST') {
        // Create the full API endpoint URL
        $url = rtrim($remote_url, '/') . '/wp-json/' . ltrim($remote_route, '/');

        // Encode the credentials for Basic Authentication
        $credentials = base64_encode($remote_user_name . ':' . $remote_wp_application_password);

        // Set up the request arguments
        $args = [
            'headers' => [
                'Authorization' => 'Basic ' . $credentials,
                'Content-Type'  => 'application/json'
            ],
            'body'    => json_encode($payload),
            'method'  => $method,
        ];

        // Send the request
        $response = \wp_remote_request($url, $args);

        // Check for errors
        if (\is_wp_error($response)) {
            return [
                'success' => false,
                'message' => $response->get_error_message()
            ];
        }

        // Check for HTTP status code errors
        $status_code = \wp_remote_retrieve_response_code($response);
        if ($status_code < 200 || $status_code >= 300) {
            return [
                'success' => false,
                'message' => 'Request failed with status code: ' . $status_code
            ];
        }

        // Decode the response body
        $response_body = \wp_remote_retrieve_body($response);

        // Return the decoded response
        return [
            'success' => true,
            'response' => \json_decode($response_body, true)
        ];
    }

    /**
     * Creates a post on a remote WordPress site.
     *
     * @param string $remote_url The base URL of the remote WordPress site.
     * @param string $remote_user_name The username for authentication.
     * @param string $remote_wp_application_password The application password for authentication.
     * @param array  $payload The post data (e.g., ['title' => 'Post Title', 'content' => 'Post content']).
     *
     * @return array Response from the remote WordPress API.
     */
    public static function create_remote_post($remote_url, $remote_user_name, $remote_wp_application_password, $payload) {
        return self::send_request_to_remote_wp_api($remote_url, $remote_user_name, $remote_wp_application_password, '/wp/v2/posts', $payload, 'POST');
    }

    /**
     * Adds metadata to a post on a remote WordPress site.
     *
     * @param string $remote_url The base URL of the remote WordPress site.
     * @param string $remote_user_name The username for authentication.
     * @param string $remote_wp_application_password The application password for authentication.
     * @param array  $payload Associative array of meta data Must contain ['post_id' => integer, 'key' => string key_name, 'value' => string value]).
     *
     * @return array Response from the remote WordPress API.
     */
    public static function add_remote_post_meta($remote_url, $remote_user_name, $remote_wp_application_password, $payload) {
        $remote_route = '/cacbot/v1/meta-data';
        return self::send_request_to_remote_wp_api($remote_url, $remote_user_name, $remote_wp_application_password, $remote_route, $payload, 'POST');
    }

    /**
     * Deletes a post from a remote WordPress site.
     *
     * @param string $remote_url The base URL of the remote WordPress site.
     * @param string $remote_user_name The username for authentication.
     * @param string $remote_wp_application_password The application password for authentication.
     * @param int    $remote_post_id The ID of the post to delete.
     *
     * @return array Response from the remote WordPress API.
     */
    public static function delete_remote_post($remote_url, $remote_user_name, $remote_wp_application_password, $remote_post_id) {
        $remote_route = '/wp/v2/cacbot-conversations/' . $remote_post_id . '?force=true';
        return self::send_request_to_remote_wp_api($remote_url, $remote_user_name, $remote_wp_application_password, $remote_route, [], 'DELETE');
    }

    /**
     * Deletes metadata from a post on a remote WordPress site.
     *
     * @param string $remote_url The base URL of the remote WordPress site.
     * @param string $remote_user_name The username for authentication.
     * @param string $remote_wp_application_password The application password for authentication.
     * @param int    $remote_post_id The ID of the post.
     * @param string $remote_meta_key The meta key to delete.
     *
     * @return array Response from the remote WordPress API.
     */
    public static function delete_remote_post_meta($remote_url, $remote_user_name, $remote_wp_application_password, $remote_post_id, $remote_meta_key) {
        $remote_route = '/wp/v2/posts/' . $remote_post_id . '/meta/' . $remote_meta_key;
        return self::send_request_to_remote_wp_api($remote_url, $remote_user_name, $remote_wp_application_password, $remote_route, [], 'DELETE');
    }
}
