<?php

namespace Cacbot;

class User
{

    /**
     * Checks if a given user is a "Cacbot".
     *
     * A user is considered a "Cacbot" if they have an email address
     * from the "cacbot.com" domain and are assigned the "cacbot" role.
     *
     * @param int $user_id The ID of the user to check.
     * @return bool True if the user is a "cacbot", false otherwise.
     */
    public static function is_user_a_cacbot($user_id)
    {
        return self::isUserFromDomainAndTLD($user_id, "cacbot.com") && self::is_user_in_role($user_id, 'cacbot');
    }

    /**
     * Checks if a user's email address is from a specified domain and TLD.
     *
     * @param int $userID The ID of the user to check.
     * @param string $domain_and_tld The domain and TLD to check against.
     * @return bool True if the user's email address is from the specified domain and TLD, false otherwise.
     */
    public static function isUserFromDomainAndTLD($userID, $domain_and_tld)
    {
        // Get user data by user ID
        $user_info = \get_userdata($userID);
        if (!$user_info) {
            // Return false if user does not exist
            return false;
        }

        // Extract the email address from user data
        $user_email = $user_info->user_email;

        // Parse the email to extract its domain and TLD
        $email_domain_and_tld = substr(strrchr($user_email, "@"), 1);

        // Compare the email's domain and tld with the input parameters
        return strtolower($email_domain_and_tld) === strtolower($domain_and_tld);
    }

    /**
     * Checks if a given user ID is associated with a specified role.
     *
     * @param int $user_id The ID of the user to check.
     * @param string $role The role to check for the user.
     * @return bool True if the user is in the specified role, false otherwise.
     */
    public static function is_user_in_role($user_id, $role)
    {
        return in_array($role, self::get_user_roles_by_user_id($user_id));
    }

    /**
     * Retrieves the roles associated with a given user ID.
     *
     * @param int $user_id The ID of the user whose roles to retrieve.
     * @return array An array of roles associated with the user.
     */
    public static function get_user_roles_by_user_id($user_id)
    {
        $user = \get_userdata($user_id);
        return empty($user) ? array() : $user->roles;
    }

    public static function enable_cacbot_role()
    {
        \add_action("init", function () {
            if (!\get_role(SLUG)) {
                \add_role(SLUG, SLUG_DISPLAY_NAME, array());
            }
        });
    }

    /**
     * There are two default Cacbot users, the Aion, and the Assistant.
     */
    public static function get_Aion_user_id()
    {
        $user = \get_user_by('email', self::get_Aion_user_email());
        if ($user === false) {
            return false;
        }
        return $user->ID;
    }

    /**
     * There are two default Cacbot users, the Aion, and the Assistant.
     */
    public static function get_Aion_user_email()
    {

        return "aion@cacbot.com";

    }

    /**
     * There are two default Cacbot users, the Aion, and the Assistant.
     */
    public static function get_cacbot_assistant_user_id()
    {
        $user = \get_user_by('email', self::get_cacbot_assistant_email());
        if ($user === false) {
            return false;
        }
        return $user->ID;
    }

    /**
     * There are two default Cacbot users, the Aion, and the Assistant.
     */
    public static function get_cacbot_assistant_email()
    {

        return "assistant@cacbot.com";

    }

    public static function generateRandomString($length = 10)
    {
        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[random_int(0, $charactersLength - 1)];
        }
        return $randomString;
    }

    /**
     * This runs upon plugin activation.
     */
    public static function do_activation_hook()
    {
        $existing_user = \get_user_by('email', self::get_cacbot_assistant_email());
        if ($existing_user) {
            return;
        }
        $Servers = new Servers();
        $endpoint = $Servers->mothershipURL . "/wp-json/cacbot-mothership/v1/remote-application-password";

        //Create the two Cacbots:
        $assistant_data = self::create_cacbot_assistant_user();
        $Aion_data = self::create_Aion_user();

        $account_owner_user_id = \get_option('_cacbot_account_owner', true);
        $account_owner_user = \get_userdata($account_owner_user_id);
        $account_owner_user_email = $account_owner_user->user_email;
        $body = [
            'cacbot_account_owner_user_email' => $account_owner_user_email,
            'users' => [$assistant_data, $Aion_data],
        ];
        $options = [
            'body' => $body,
            'timeout' => 60,
            'redirection' => 5,
            'blocking' => true,
            'httpversion' => '1.0',
            'sslverify' => false,
        ];
        $result = \wp_remote_post($endpoint, $options);

    }

    public static function create_cacbot_assistant_user()
    {
        $username = "Assistant";
        while (\username_exists($username)) {
            $username = substr(str_shuffle("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"), 0, 10);
        }
        $password = \wp_generate_password();
        $website = "https://cacbot.com";
        $userdata = array(
            'user_login' => $username,
            'user_url' => $website,
            'user_pass' => $password,
            'user_email' => self::get_cacbot_assistant_email(),
        );
        $user_id = \wp_insert_user($userdata);
        $user = new \WP_User($user_id);
        $user->set_role('editor');
        \update_user_meta($user_id, 'first_name', 'Carlito');
        \update_user_meta($user_id, 'last_name', 'Young');
        \update_user_meta($user_id, 'description', 'I am a Cacbot. Get skills for your Cacbot at https://cacbot.com .');
        \update_user_meta($user_id, 'url', 'https://cacbot.com');
        self::assign_cacbot_role_to_user($user_id);
        $app_password_name = 'Cacbot';
        $body = [];
        $pw = \WP_Application_Passwords::create_new_application_password($user_id, array('name' => $app_password_name));
        $pw = $pw[0];
        $body[] = ['app-password' => $pw];
        $body[] = ["userName" => $username];
        $body[] = ["email" => "assistant@cacbot.com"];
        $body[] = ["url" => \get_site_url()];
        $body[] = ["hashed_pw" => User::get_hashed_application_password($user_id, "Cacbot")];
        return $body;
    }

    public static function assign_cacbot_role_to_user($user_id)
    {
        // Check if user exists
        $user = \get_user_by('ID', $user_id);
        if ($user) {
            // Add 'aion' role to existing roles
            $user->add_role('cacbot');
        }
    }

    public static function get_hashed_application_password($user_id, $app_password_name)
    {
        // Ensure the user ID is valid
        if (!is_int($user_id) || $user_id <= 0) {
            return null;
        }

        // Retrieve all stored application passwords for the user
        $app_passwords = get_user_meta($user_id, '_application_passwords', true);
        //var_dump($app_passwords);
        if (empty($app_passwords)) {
            return null; // No application passwords found for this user
        }

        // Loop through the application passwords to find the one that matches our name
        foreach ($app_passwords as $app_password) {
            if ($app_password['name'] === $app_password_name) {
                // Return the hashed password associated with the specified name
                return $app_password['password'];
            }
        }

        return null; // Password with the specified name not found
    }

    public static function create_Aion_user()
    {
        $username = "Aion";
        while (\username_exists($username)) {
            $username = substr(str_shuffle("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"), 0, 10);
        }
        $password = \wp_generate_password();
        $website = "https://cacbot.com";
        $userdata = array(
            'user_login' => $username,
            'user_url' => $website,
            'user_pass' => $password,
            'user_email' => self::get_Aion_user_email(),
        );
        $user_id = \wp_insert_user($userdata);
        $user = new \WP_User($user_id);
        $user->set_role('editor');
        \update_user_meta($user_id, 'first_name', 'Peter');
        \update_user_meta($user_id, 'last_name', 'Chardin');
        \update_user_meta($user_id, 'description', 'I am a Cacbot. Get skills for your Cacbot at https://cacbot.com .');
        \update_user_meta($user_id, 'user_url', 'https://cacbot.com');
        //\wp_new_user_notification($user_id, null, 'both');
        self::assign_cacbot_role_to_user($user_id);
        $app_password_name = 'Cacbot';
        $body = [];
        $pw = \WP_Application_Passwords::create_new_application_password($user_id, array('name' => $app_password_name));
        $pw = $pw[0];
        $body[] = ['app-password' => $pw];
        $body[] = ["userName" => $username];
        $body[] = ["email" => "aion@cacbot.com"];
        $body[] = ["url" => \get_site_url()];
        $body[] = ["hashed_pw" => User::get_hashed_application_password($user_id, "Cacbot")];
        return $body;

    }

    public static function sendApplicationPasswordToMothership($user_id, $password)
    {

        \wp_remote_post("", []);
    }

    public static function does_aion_assistant_user_exist()
    {
        $user = \get_user_by('email', "assistant@cacbot.com");
        if ($user === false) {
            return false;
        }
        return $user->ID;
    }

    public static function does_Aion_user_exist()
    {
        $user = \get_user_by('email', "aion@cacbot.com");
        if ($user === false) {
            return false;
        }
        return $user->ID;
    }

}